@@ -1,6 +1,8 @@
 import json
 import logging
 import requests
+import socket
+import ssl
 from urllib.parse import urljoin
 
 from tabulate import tabulate
@@ -8,7 +10,7 @@ from tabulate import tabulate
 from waiter import http_util, terminal
 from waiter.format import format_last_request_time
 from waiter.format import format_status
-from waiter.querying import get_service, get_services_using_token
+from waiter.querying import get_service, get_service_id_from_instance_id, get_services_using_token
 from waiter.querying import print_no_data, query_service, query_services, query_token
 from waiter.util import is_service_current, str2bool, response_message, print_error, wait_until
 
@@ -91,6 +93,17 @@ def ping_on_cluster(cluster, timeout, wait_for_request, token_name, service_exis
     return succeeded
 
 
+def check_ssl(token_name, port, timeout_seconds):
+    """Returns true when a socket to the token's DNS name doesn't raise an SSL or connection error"""
+    timeout_millis = timeout_seconds * 1000
+    context = ssl.create_default_context()
+    with socket.create_connection((token_name, port)) as cleartext_socket:
+        with context.wrap_socket(cleartext_socket, server_hostname=token_name) as tls_socket:
+            tls_version = tls_socket.version()
+            print(f'Successfully connected to {token_name}:{port} using {tls_version}')
+    return True
+
+
 def token_has_current_service(cluster, token_name, current_token_etag):
     """If the given token has a "current" service, returns that service else None"""
     services = get_services_using_token(cluster, token_name)
@@ -287,3 +300,69 @@ def process_ping_request(clusters, token_name_or_service_id, is_service_id, time
                 success = True
         overall_success = overall_success and success
     return overall_success
+
+
+def send_signal_to_instance_on_cluster(cluster, service_id, instance_id, signal_type, query_params):
+    """Send sigkill request to the specific instance"""
+    cluster_name = cluster['name']
+    http_util.set_retries(0)
+    try:
+        print(
+            f'Sending {terminal.bold(signal_type)} request to instance {terminal.bold(instance_id)} in {terminal.bold(cluster_name)}...')
+        resp = http_util.post(cluster, f'/apps/{service_id}/instance/{instance_id}/{signal_type}', '',
+                              params=query_params)
+        logging.debug(f'Response status code: {resp.status_code}')
+        if resp.status_code == 200:
+            resp_json = resp.json()
+            logging.debug(f'Response json: {resp_json}')
+            signal_response = resp_json.get("signal-response", {})
+            success = signal_response.get('success')
+            if success:
+                print(f'Successfully sent {signal_type} to {instance_id} in {cluster_name}.')
+                return True
+            else:
+                message = signal_response.get('message')
+                print(f'Unable to send {signal_type} to {instance_id} in {cluster_name}. {message}')
+                return False
+        else:
+            print_error(response_message(resp.json()))
+            return False
+    except requests.exceptions.ReadTimeout:
+        message = f'Request timed out while signaling {instance_id} in {cluster_name}.'
+        logging.exception(message)
+        print_error(message)
+        return False
+    except Exception:
+        message = f'Encountered error while signaling {instance_id} in {cluster_name}.'
+        logging.exception(message)
+        print_error(message)
+        return False
+
+
+def process_signal_request(clusters, instance_id, signal_type, query_params, no_instance_result=False):
+    """Sends signal to the instance given the instance id.
+    Returns False if there are no instances or if there is no active instance with the same instance id
+    Returns True if the signal request is sent successfully."""
+    service_id = get_service_id_from_instance_id(instance_id)
+    query_result = query_service(clusters, service_id)
+    num_services = query_result['count']
+    if num_services == 0:
+        print_no_data(clusters)
+        return no_instance_result
+
+    cluster_data_pairs = sorted(query_result['clusters'].items())
+    clusters_by_name = {c['name']: c for c in clusters}
+    for cluster_name, data in cluster_data_pairs:
+        service = data['service']
+        if service['status'] == 'Inactive':
+            logging.debug(f"Skipping {service['status']} service {service_id} on {cluster_name} cluster")
+            continue
+
+        active_instances = service.get('instances', {}).get('active-instances')
+        for instance in active_instances:
+            if instance['id'] == instance_id:
+                cluster = clusters_by_name[cluster_name]
+                return send_signal_to_instance_on_cluster(cluster, service_id, instance_id, signal_type, query_params)
+
+    print(f'No active instance with ID {terminal.bold(instance_id)}')
+    return False
